<?php

namespace Ems\Controllers;

use Exception\Messages;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Doctrine\ORM\EntityManager;

use Ems\Services\ProcessoService;

class ProcessoController {
    
    /** @var ProcessoService */
    protected $processoService;
    
    public function __construct(
        ProcessoService $processoService
    ) {
        $this->processoService = $processoService;
    }
    
    public function getStatusProcesso(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            
            $status = $this->processoService->getStatusProcesso();

            $response->addDataSet(new DataSet('getStatusProcesso', $status));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getProcessosGestor(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrParcNegocio = (int) $row['NRPARCNEGOCIO'];

            $processos = $this->processoService->getProcessosGestor($nrParcNegocio);

            $response->addDataSet(new DataSet('getProcessosGestor', $processos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getFasesTemplate(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrTemplate = (int) $row['NRTEMPLATE'];

            $fases = $this->processoService->getFasesTemplate($nrTemplate);

            $response->addDataSet(new DataSet('getFasesTemplate', $fases));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getCandidatosFase(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrProcesso = (int) $row['NRPROCESSO'];
            $nrFase = (int) $row['NRFASE'];

            $candidatos = $this->processoService->getCandidatosFase(
                $nrProcesso, $nrFase
            );

            $response->addDataSet(new DataSet('getCandidatosFase', $candidatos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}
