<?php

namespace Ems\Repository;

use Ems\Util\Utils;
use Doctrine\ORM\EntityManager;

class ProcessoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getStatusProcesso(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
              CDSTATUS,
              DSSTATUS
              FROM DES_STATUS
             WHERE (NRORG = :P_NRORG_PADRAO OR NRORG = :P_NRORG)
             ORDER BY DSSTATUS
SQL;

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std = Utils::getEntityManager()->getConnection()->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getProcessosGestor(
        int $nrOrg,
        int $nrParcNegocio
    ): array {
        $sql = <<<SQL
            SELECT
                PRO.*,
                TO_CHAR(PRO.DTLIMSOLICITA, 'DD/MM/YYYY') DTLIMSOLICITA,
                TO_CHAR(PRO.DTSOLICITA, 'DD/MM/YYYY') DTSOLICITA,
                RESP.NMPRINCIPALPARC NMRESPONSAVEL,
                SOLI.NMPRINCIPALPARC NMSOLICITA,
                DES_TEMPLATE.NMTEMPLATE,
                DES_STATUS.DSSTATUS,
                OCUPA.NMOCUPACAOH,
                ESTRUTURAH.NMESTRUTURAH,
                ESCALA.NMESCALATRABH,
                DES_FASE.NRFASE,
                DES_FASE.NMFASE
            FROM
                DES_PROCESSO PRO
                LEFT JOIN DES_TIPOPROCESSO TIPO ON
                    TIPO.NRTIPOPROCESSO = PRO.NRTIPOPROCESSO AND
                    TIPO.NRORG = PRO.NRORG
                LEFT JOIN PARCNEGOCIO RESP ON
                    RESP.NRPARCNEGOCIO = PRO.NRPARCNEGOCIORESP AND
                    RESP.NRORG = PRO.NRORG
                LEFT JOIN PARCNEGOCIO SOLI ON
                    SOLI.NRPARCNEGOCIO = PRO.NRPARCNEGOCIOSOLICITA AND
                    SOLI.NRORG = PRO.NRORG
                LEFT JOIN DES_TEMPLATE ON
                    DES_TEMPLATE.NRTEMPLATE = PRO.NRTEMPLATE AND
                    DES_TEMPLATE.NRORG = PRO.NRORG
                LEFT JOIN DES_STATUS ON
                    DES_STATUS.CDSTATUS = PRO.IDSTATUS
                LEFT JOIN GPE_OCUPACAOH OCUPA ON
                    OCUPA.DTMESCOMPETENC = (
                        SELECT MAX(OC1.DTMESCOMPETENC)
                        FROM GPE_OCUPACAOH OC1
                        WHERE
                            OC1.DTMESCOMPETENC <= SYSDATE AND
                            OC1.NROCUPACAOM = PRO.NROCUPACAODESEJADA AND
                            OC1.NRORG = PRO.NRORG
                    ) AND
                    OCUPA.NROCUPACAOM = PRO.NROCUPACAODESEJADA AND
                    OCUPA.NRORG = PRO.NRORG
                LEFT JOIN ESTRUTURAH ON
                    ESTRUTURAH.DTMESCOMPETENC = (
                        SELECT MAX(E1.DTMESCOMPETENC)
                        FROM ESTRUTURAH E1
                        WHERE
                            E1.DTMESCOMPETENC <= SYSDATE AND
                            E1.NRESTRUTURAM = PRO.NRESTRUTGEREN AND
                            E1.NRORG = PRO.NRORG
                    ) AND
                    ESTRUTURAH.NRESTRUTURAM = PRO.NRESTRUTGEREN AND
                    ESTRUTURAH.NRORG = PRO.NRORG
                LEFT JOIN GPE_ESCALATRABH ESCALA ON
                    ESCALA.DTMESCOMPETENC = (
                        SELECT MAX(ESC1.DTMESCOMPETENC)
                        FROM GPE_ESCALATRABH ESC1
                        WHERE
                            ESC1.DTMESCOMPETENC <= SYSDATE AND
                            ESC1.NRESCALATRABM = PRO.NRESCALATRABM AND
                            ESC1.NRORG = PRO.NRORG
                    ) AND
                    ESCALA.NRESCALATRABM = PRO.NRESCALATRABM AND
                    ESCALA.NRORG = PRO.NRORG
                LEFT JOIN DES_FASE ON
                    DES_FASE.NRFASE = (
                        SELECT MAX(F1.NRFASE)
                        FROM
                            DES_FASE F1
                            JOIN DES_PARTICIPAFASE PF1 ON
                                PF1.NRFASE = F1.NRFASE AND
                                PF1.NRPROCESSO = PRO.NRPROCESSO AND
                                PF1.NRORG = F1.NRORG
                    ) AND
                    DES_FASE.NRORG = PRO.NRORG
            WHERE
                :P_NRPARC IN (PRO.NRPARCNEGOCIOSOLICITA, PRO.NRPARCNEGOCIORESP) AND
                PRO.NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARC' => $nrParcNegocio,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();

        return $result;
    }

    public function getFasesTemplate(
        int $nrOrg,
        int $nrTemplate
    ): array {
        $sql = <<<SQL
            SELECT
                IDINATIVARPROCESSO,
                NRORDEM,
                NRFASETEMPLATE,
                FT.NRORG,
                FT.NRTEMPLATE,
                FT.NRFASE,
                F.NMFASE,
                FT.NRDURACAO,
                FT.NRPESO,
                FT.IDOBRIGATORIO,
                NRFASERELACIONADA,
                (
                    SELECT
                        NMFASE
                    FROM
                        DES_FASE
                    WHERE
                            NRORG = FT.NRORG
                        AND NRFASE = NRFASERELACIONADA
                ) AS NMFASERELACIONADA,
                F.DTINCLUSAO,
                F.DTULTATU,
                F.NRORGINCLUSAO,
                F.CDOPERINCLUSAO,
                F.NRORGULTATU,
                F.CDOPERULTATU,
                PF.NRCRITERIORESUL,
                CR.NMCRITERIORESUL,
                PF.VRFASE,
                (
                    SELECT
                        MAX(NRORDEM)
                    FROM
                        DES_FASETEMPLATE
                    WHERE
                            NRTEMPLATE = FT.NRTEMPLATE
                        AND NRORG = FT.NRORG
                ) AS MAXNRORDEM
            FROM
                    DES_FASETEMPLATE FT
                JOIN DES_FASE       F ON FT.NRFASE = F.NRFASE
                                AND FT.NRORG = F.NRORG
                JOIN DES_TEMPLATE   T ON FT.NRTEMPLATE = T.NRTEMPLATE
                                    AND FT.NRORG = T.NRORG
                LEFT JOIN DES_PARAMFASE  PF ON FT.NRFASE = PF.NRFASE
                                            AND FT.NRORG = PF.NRORG
                LEFT JOIN CRITERIORESULT CR ON PF.NRCRITERIORESUL = CR.NRCRITERIORESUL
                                            AND FT.NRORG = CR.NRORG
            WHERE
                    FT.NRTEMPLATE = :P_NRTEMPLATE
                AND FT.NRORG = :P_NRORG
            ORDER BY
                NRORDEM
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRTEMPLATE' => $nrTemplate,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();

        return $result;
    }

    public function getCandidatosFase(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                PF.NRORG,
                PF.NRPARCNEGOCIOCAND,
                PF.NRPARCTICIPAFASE,
                PF.NRFASE,
                PF.NRPROCESSO,
                PF.DTINICIO,
                PF.DTFIM,
                PF.NRPONTUACAO,
                PF.NRAVALIACAO,
                PF.NRDURACAO,
                PF.NRPESO,
                PF.IDOBRIGATORIO,
                PF.DSLOCALFASE,
                PF.IDSTATUS,
                PF.DTINCLUSAO,
                PF.DTULTATU,
                PF.NRORGINCLUSAO,
                PF.CDOPERINCLUSAO,
                PF.NRORGULTATU,
                PF.CDOPERULTATU,
                S.DSSTATUS,
                P.NMPRINCIPALPARC,
                PF.CDTIPORESU,
                TP.NMTIPORESU,
                DECODE(PF.NRPERCENTUAL, NULL, NULL, PF.NRPERCENTUAL || '%')       NRPERCENTUAL,
                PESSOAH.NRCPFPESSOA,
                DECODE(PC.VRMEDIAPROCESSO, NULL, NULL, PC.VRMEDIAPROCESSO || '%') VRMEDIAPROCESSO
            FROM
                    DES_PARTICIPAFASE PF
                JOIN PARCNEGOCIO           P ON PF.NRPARCNEGOCIOCAND = P.NRPARCNEGOCIO
                                    AND P.NRORG = :P_NRORGPADRAO
                LEFT JOIN DES_STATUS            S ON PF.IDSTATUS = S.CDSTATUS
                LEFT JOIN TIPORESU              TP ON PF.CDTIPORESU = TP.CDTIPORESU
                                        AND PF.NRORG = TP.NRORG
                LEFT JOIN GPE_PESSOA            PESSOA ON PESSOA.NRPARCNEGOCIO = PF.NRPARCNEGOCIOCAND
                                            AND PESSOA.NRORG = P.NRORG
                LEFT JOIN GPE_PESSOAH           PESSOAH ON PESSOAH.NRPESSOA = PESSOA.NRPESSOA
                                                AND PESSOAH.NRORG = PESSOA.NRORG
                LEFT JOIN DES_PARTICIPAPROCESSO PC ON PC.NRPARCNEGOCIOCAND = PF.NRPARCNEGOCIOCAND
                                                    AND PC.NRPROCESSO = PF.NRPROCESSO
                                                    AND PC.NRORG = PF.NRORG
            WHERE
                    PF.NRORG = :P_NRORG
                AND PF.NRPROCESSO = :P_NRPROCESSO
                AND PF.NRFASE = :P_NRFASE
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();

        return $result;
    }
}