<?php

namespace Ems\Services;

use Util\Environment;
use Ems\Repository\{
    HierarquiaRepository,
    VinculoRepository
};
use Ems\Services\{
    EstruturaService
};
use Util\NovoCodigo;

class HierarquiaService {
    /** @var Environment */
    protected $environment;

    /** @var HierarquiaRepository */
    protected $hierarquiaRepository;

    /** @var VinculoRepository */
    protected $vinculoRepository;

    /** @var EstruturaService */
    protected $estruturaService;

    public function __construct(
        Environment             $environment,
        HierarquiaRepository    $hierarquiaRepository,
        VinculoRepository       $vinculoRepository,
        EstruturaService        $estruturaService
    ) {
        $this->environment          = $environment;
        $this->hierarquiaRepository = $hierarquiaRepository;
        $this->vinculoRepository    = $vinculoRepository;
        $this->estruturaService     = $estruturaService;
    }

    public function getTiposHierarquia(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $result = $this->hierarquiaRepository->getTiposHierarquia($params);

        return $result;
    }

    public function getHierarquiaOrganizacao(
        int $nrVinculo
    ) {
        $estruturaRaiz = $this->getEstruturaRaiz($nrVinculo);
        $hierarquia = $this->getHierarquiaEstrutura($estruturaRaiz);

        return $hierarquia;
    }

    public function getEstruturaRaiz(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $vinculo = $this->vinculoRepository->getVinculoHistorico(
            $nrOrg,
            $nrVinculo
        );

        $nrEstrutura = $vinculo['NRESTRUTLEGAL'];

        if ($nrEstrutura === null) {
            throw new \Exception('Nenhuma estrutura legal associada ao vínculo!');
        }

        $estrutura = $this->estruturaService->getDadosEstrutura($nrEstrutura);

        return $estrutura;
    }

    public function getHierarquiaEstrutura(
        array $estrutura
    ): array {
        $nrEstrutura = (int) $estrutura['NRESTRUTURAM'];
        $nrsEstruturasInferiores = $this->estruturaService->getNrsEstruturasInferiores($nrEstrutura);
        $estruturasInferiores = [];

        foreach ($nrsEstruturasInferiores as $nrEstruturaInferior) {
            $estruturaInferior = $this->estruturaService->getDadosEstrutura($nrEstruturaInferior);
            $estruturasInferiores[] = $this->getHierarquiaEstrutura($estruturaInferior);
        }

        $estrutura['__children'] = $estruturasInferiores;

        return $estrutura;
    }
}