<?php

namespace Ems\Services;

use Util\Environment;
use Ems\Repository\MovimentacaoRepository;
use Util\NovoCodigo;

class MovimentacaoService {
    /** @var Environment */
    protected $environment;

    /** @var MovimentacaoRepository */
    protected $movimentacaoRepository;

    public function __construct(
        Environment             $environment,
        MovimentacaoRepository  $movimentacaoRepository
    ) {
        $this->environment              = $environment;
        $this->movimentacaoRepository   = $movimentacaoRepository;
    }

    public function setMovimentacao(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRMOVIMENTACAO'] =
            $row['NRMOVIMENTACAO'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_MOVIMENTACAO', 1, 12, true, false);

        $params['NRORG']            = $row['NRORG'] ?? $nrOrg;
        $params['NRVINCULOM']       = (int) $row['NRVINCULOM'];
        $params['NRTIPOTRANSFER']   = (int) $row['NRTIPOTRANSFER'];
        $params['NRTPMOVTRANSFM']   = $row['NRTPMOVTRANSFM'] ?? null;
        $params['NRMOTIVOTRANSF']   = $row['NRMOTIVOTRANSF'] ?? null;
        $params['NRESTRUTURAM']     = (int) $row['NRESTRUTURAM'];
        $params['DTINIMOVIMENT']    = (string) $row['DTINIMOVIMENT'];
        $params['DTFIMMOVIMENT']    = $row['DTFIMMOVIMENT'] ?? null;
        $params['DSOBSERVACAO']     = $row['DSOBSERVACAO'] ?? null;
        $params['DTINCLUSAO']       = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']         = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']    = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']   = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']      = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']     = $row['CDOPERULTATU'] ?? null;
        $params['NRTIPOESTRUTURA']  = $row['NRTIPOESTRUTURA'] ?? null;
        $params['DTMOVIMENTRETRO']  = $row['DTMOVIMENTRETRO'] ?? null;
        $params['IDATUALIZAACERTO'] = $row['IDATUALIZAACERTO'] ?? 0;
        $params['NRMOTIVOAFASTA']   = $row['NRMOTIVOAFASTA'] ?? null;

        $this->movimentacaoRepository->setMovimentacao($params);

        return $params['NRMOVIMENTACAO'];
    }
}