<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Ems\Repository\ProcessoRepository;
use Ems\Util\Utils;
use Util\Environment;

class ProcessoService {
    /** @var ProcessoRepository */
    protected $processoRepository;

    /** @var Environment */
    protected $environment;

    public function __construct(
        ProcessoRepository  $processoRepository,
        Environment $environment
    ) {
        $this->processoRepository = $processoRepository;
        $this->environment = $environment;
    }

    public function getStatusProcesso(): array {
        return $this->processoRepository->getStatusProcesso(
            Utils::getParamters()->getNrorg(),
            Utils::getParamters()->getNrorgPadrao()
        );
    }

    public function getProcessosGestor(
        int $nrParcNegocio
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $processos = $this->processoRepository->getProcessosGestor(
            $nrOrg, $nrParcNegocio
        );

        return $processos;
    }

    public function getFasesTemplate(
        int $nrTemplate
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $fases = $this->processoRepository->getFasesTemplate(
            $nrOrg, $nrTemplate
        );

        return $fases;
    }

    public function getCandidatosFase(
        int $nrProcesso,
        int $nrFase
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORGPADRAO' => $nrOrgPadrao,
            'P_NRPROCESSO' => $nrProcesso,
            'P_NRFASE' => $nrFase,
        ];

        $candidatos = $this->processoRepository->getCandidatosFase(
            $params
        );

        return $candidatos;
    }
}


